#include "db.h"

#include "log.h"

#include <mysql/mysql.h>

// printf()
#include <stdio.h>

// str*()
#include <string.h>

// free()
#include <stdlib.h>

// internal linkage - only in this translation unit
static MYSQL *handler = NULL;
static char *dbhost = NULL, *dbuser = NULL, *dbpass = NULL, *dbname = NULL;

void db_error_log(void)
{
	log_stamp();
	fprintf(logfile, "mysql_errno: %u\n", mysql_errno(handler));
	log_stamp();
	fprintf(logfile, "mysql_error: %s\n", mysql_error(handler));
}

void db_close(void)
{
	if (handler != NULL) {
		//log_write("calling myslq_close()\n");
		mysql_close(handler);
		// to the beginning
		handler = NULL;
	}
}

int db_connect(void)
{
	// test if variables have values
	if (!(dbhost && dbuser && dbpass && dbname)) {
		log_write("db_connect() failed; call db_init() first\n");
		// bad, change to 1 or NULL later
		return 0;
	}

	if (handler == NULL) {
		//fputs("handler is NULL, calling mysql_init()\n", logfile);
		if ((handler = mysql_init(NULL)) == NULL) {
			log_write("mysql_init() failed\n");
			return 0;
		}
	}
	// calling mysql_ping() here leads to segfault (known error)
	// and reconnect (and thread_id() and...) behavior depends on versions
	// so avoid this whole thing and close in case of error
	if (mysql_real_connect(handler, dbhost, dbuser, dbpass, dbname,
				0, NULL, 0) == NULL) {
		log_write("mysql_real_connect() failed\n");
		db_error_log();
		// cleanup
		db_close();
		return 0;
	}
	return 1;
}

int db_query(const char *query)
{
	// db is not connected
	if (! handler) {
		db_connect();
		// something failed, can not connect
		if (! handler) return 0;
	}

	if (mysql_query(handler, query)) {
		log_stamp();
		fprintf(logfile, "mysql_query(%s) failed\n", query);
		db_error_log();
		// cleanup
		db_close();
		return 0;
	}

	return 1;
}

int db_init(char *host, char *user, char *passwd, char *db)
{
	if ((dbhost = strdup(host))) {
		if ((dbuser = strdup(user)))
			if ((dbpass = strdup(passwd)))
				if ((dbname = strdup(db)))
					return 0;
				else
					free(dbpass);
			else
				free(dbuser);
		else
			free(dbhost);
	}
	return 1;
}

void db_finish(void)
{
	db_close();
	free(dbhost);
	free(dbuser);
	free(dbpass);
	free(dbname);
}

long long db_count_rows(const char *table)
{
	char q_begin[] = "select count(*) from ";
	// + 1 for \0
	char query[strlen(q_begin) + strlen(table) + 1];
	MYSQL_RES *result;
	MYSQL_ROW row;
	long long count;

	// create query
	sprintf(query, "%s%s", q_begin, table);

	// query failed
	if (! db_query(query)) return -1;

	// no result ~ error (which?)
	if (! (result = mysql_store_result(handler))) return -1;

	// empty row?
	if (! (row = mysql_fetch_row(result))) return -1;

	// store value before calling free
	count = atoll(row[0]);

	mysql_free_result(result);

	return count;
}

MYSQL_RES *db_query_store(const char *query)
{
	MYSQL_RES *result;

	// query failed
	if (! db_query(query)) return NULL;

	if (! (result = mysql_store_result(handler)))
		// no result
		if (mysql_field_count(handler)) {
			// store should have been non empty, so error happened
			log_write("mysql_store_result failed\n");
			db_error_log();
			// cleanup
			db_close();
		}

	return result;
}

char *db_get_one(const char *query)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	char *string;

	// exit if error
	if (! (result = db_query_store(query))) return NULL;

	if ((row = mysql_fetch_row(result)))
		// store value before calling free
		string = strdup(row[0]);
	else
		// empty row?
		string = NULL;

	mysql_free_result(result);

	return string;
}
